# coding=utf-8
"""
脱离tfduck系统环境的一些公共方法
"""
from tfduck.common.extendEncoder import DatetimeJSONEncoder
from tfduck.common.defines import Et, CeleryRetryError
import ast
import hashlib
import json
import logging
import sys
import shutil
import time
import traceback
import random
import requests
import os
import uuid
import arrow



class BaseMethod(object):
    """
    @des:一些基础方法
    """

    def __init__(self):
        """
        @des:保持一些全局变量
        """
        self.dj44_base_dir = "/opt/djcelery44/djcelery44"
        self.http_api = Dj44HttpApi() # 后面使用需要BMOBJ.http_api.set_user_token('xxxxx')设置token
    
    def get_unique_id(self):
        unique_id = (uuid.uuid5(uuid.NAMESPACE_DNS, str(uuid.uuid1()) + str(random.random()))).hex
        return unique_id
    
    def get_unique_id2(self):
        unique_id = (uuid.uuid5(uuid.NAMESPACE_DNS, str(uuid.uuid1()) + str(random.random()))).hex
        return unique_id

    def remove_file(self, file_path):
        """
        @des: 删除文件
        """
        try:
            os.remove(file_path)
        except:
            pass

    def remove_folder(self, folder_path):
        """
        @des: 删除文件夹
        """
        try:
            shutil.rmtree(folder_path)
        except:
            pass

    def get_file_path(self, path):
        """
        @des: 文件的真实路径
        """
        return os.path.join(self.dj44_base_dir, os.path.join("dags/sptasks/p_code", path))
        
            

    def gen_local_unique_file(self, pre_path="", pre_name="", ext="csv"):
        """
        @des:生成本地文件唯一路径
        """
        # media_root = settings.MEDIA_ROOT
        media_root = "/mydata/media"
        base_dir = os.path.join(media_root, "docs", pre_path)
        if not os.path.exists(base_dir):
            os.makedirs(base_dir)
        real_name = "%s%s_%s.%s" % (pre_name, uuid.uuid1(
        ).hex, arrow.now().format("YYYYMMDD_HHmmSS"), ext)
        file_path = os.path.join(base_dir, real_name)
        return file_path

    def gen_local_unique_path(self, pre_path="", pre_name="", is_create=False):
        """
        @des:生成本地目录唯一路径
        """
        # media_root = settings.MEDIA_ROOT
        media_root = "/mydata/media"
        base_dir = os.path.join(media_root, "docs", pre_path)
        if not os.path.exists(base_dir):
            os.makedirs(base_dir)
        real_name = "%s%s_%s" % (pre_name, uuid.uuid1(
        ).hex, arrow.now().format("YYYYMMDD_HHmmSS"))
        folder_path = os.path.join(base_dir, real_name)
        if is_create:
            os.makedirs(folder_path)
        return folder_path

    def get_var(self, name):
        """
        @des: 获取变量
        """
        data = self.http_api.get_model_data("gconfig", "VarConfig", {"name": name}, ["content"])
        result = json.loads(data[0]['content'])
        return result

    def clog(self, ctx, *log_values):
        """
        @des: 向数据库中记录日志
        """
        try:
            pd_logs = [str(item) for item in log_values]
            pd_logs = "".join(pd_logs)
            if len(pd_logs) > 1024*1024*50:  # 大于5M
                log_values = ["log data too long"]
        except:
            pass
        
        task_type = ctx['task_type']
        trid = ctx['trid']
        index = ctx['index']
        #
        real_values = []
        for log_value in log_values:
            try:
                value = str(log_value)
                real_values.append(value)
            except:
                value = "log value must be string"
                real_values.append(value)

        app_name = None
        if task_type == "dtask":
            # Record = apps.get_model("dtask", 'RunRecord')
            app_name = "dtask"
            model_name = "RunRecord"
        elif task_type == "retask":
            # Record = apps.get_model("retask", 'RETaskRecord')
            app_name = "retask"
            model_name = "RETaskRecord"
        elif task_type == "sptask":
            # Record = apps.get_model("sptask", 'DPTaskRecord')
            app_name = "sptask"
            model_name = "DPTaskRecord"
        if app_name is not None:
            try:
                self.http_api.run_model_method(app_name, model_name, "add_records", {
                    "lock_id": trid, "task_index": index, "recs": real_values})
                # Record.objects.add_records(
                #     lock_id=trid, task_index=index, recs=real_values)
            except Exception as e:
                self.log_error("clog error ----- :")
                self.logerr(e)
        # 防止过度写入日志
        time.sleep(0.2)

    def get_record_now(self, ctx={}, tz="UTC"):
        """
        @des: 获取执行任务记录的创建时间，方便dag里面取根据现在时间去执行part_date-----其实这个方法用不到，因为都是在dag里面用,dag用的是defines.BMOBJ，因为这个dag是必须带默认环境的
        start = arrow.now(tz="Asia/Shanghai")
        @ return: datetime 带时区
        """
        # 如果是查看任务，而不是任务记录，是不会有这两个参数的
        task_type = ctx.get('task_type', None)
        trid = ctx.get('trid', None)
        if trid is None:
            root_create_time = arrow.now(tz=tz)
        else:
            # Record = None
            if task_type == "dtask":
                # Record = apps.get_model("dtask", 'RunRecord')
                app_name = "dtask"
                model_name = "RunRecord"
            elif task_type == "retask":
                # Record = apps.get_model("retask", 'RETaskRecord')
                app_name = "retask"
                model_name = "RETaskRecord"
            elif task_type == "sptask":
                # Record = apps.get_model("sptask", 'DPTaskRecord')
                app_name = "sptask"
                model_name = "DPTaskRecord"
            if app_name is not None:
                # obj = Record.objects.get(id=trid)
                objs = self.http_api.get_model_data(app_name, model_name, {"id": trid}, ["extra", "create_time"])
                #
                obj = objs[0]
                obj['extras'] = json.loads(obj['extra'])
                #
                root_create_time = obj['extras'].get("root_create_time", None)
                if root_create_time is None:  # 如果拿不到，那么obj肯定是根节点
                    root_create_time = arrow.get(obj['create_time']).to(tz)
                else:  # 读取extra的信息，即根节点
                    # root_create_time带有时区信息，可以直接get
                    root_create_time = arrow.get(root_create_time).to(tz)
        # 返回
        return root_create_time.datetime

    def jsonloads(self, data):
        """
        @des: 自动jsonload
        """
        try:
            data = json.loads(data)
        except Exception as e:
            try:
                data = ast.literal_eval(data)
            except Exception as e:
                raise Et(2, u"参数格式不正确")
        return data

    def jsondumps(self, data, mode=1, sort_keys=False):
        """
        @des:返回值dumps
        """
        #data = json.dumps(data, ensure_ascii=False, cls=DatetimeJSONEncoder)
        data = json.dumps(data, separators=(',', ':'), cls=DatetimeJSONEncoder)
        return data

    def text_tran_html(self, text):
        """
        @des:将文本转换为html,比如help_text
        """
        text = text.replace("\n", "<br/>")
        text = text.replace(" ", "&nbsp")
        return text


    def geterr(self, e):
        result = []
        errorMeg = ''
        try:
            for file, lineno, function, text in traceback.extract_tb(sys.exc_info()[2]):
                errorMeg += '%s\n%s, in %s\n%s:                %s!' % (
                    str(e), file, function, lineno, text)
            result.append("error"+"*"*50)
            for error in errorMeg.split("\n"):
                result.append(error)
                result.append('error')
            try:
                result.append(getattr(e, '_msg', 'exception'))
            except Exception as e1:
                result.append(getattr(e, 'msg', 'exception'))
        except Exception as e2:
            result.append(str(e2))
        return "\n".join(result)

    def logerr(self, e):
        errorMeg = ''
        try:
            for file, lineno, function, text in traceback.extract_tb(sys.exc_info()[2]):
                errorMeg += '%s\n%s, in %s\n%s:                %s!' % (
                    str(e), file, function, lineno, text)
            self.log_error("error"+"*"*50)
            for error in errorMeg.split("\n"):
                self.log_error(error, "error")
            try:
                error_self = "".join(traceback.format_exception_only(type(e), e))  # 捕获错误本身,这样才能捕获到compile里面的错误
                self.log_error(error_self, "error")
            except:
                pass
            try:
                self.log_error(getattr(e, '_msg', 'exception'))
            except Exception as e1:
                self.log_error(getattr(e, 'msg', 'exception'))
        except Exception as e2:
            self.log_error(e2)

    def get_log_str(self, *msgs):
        try:
            msg_str = " ".join([str(msg) for msg in msgs])
        except Exception as _:
            msg_str = msgs
        return msg_str

    def log_debug(self, *msgs):
        msg_str = self.get_log_str(*msgs)
        print(msg_str)

    def log_info(self, *msgs):
        msg_str = self.get_log_str(*msgs)
        print(msg_str)

    def log_error(self, *msgs):
        msg_str = self.get_log_str(*msgs)
        print(msg_str)

    def log_warning(self, *msgs):
        msg_str = self.get_log_str(*msgs)
        print(msg_str)
    # end--日志记录


class Dj44HttpApi(object):
    """
    @des: 外部api
    """
    def __init__(self):
        self.r_timeout = (5, 10)
        self.retry_count = 1
        self.retry_delay = 5
        self.user_token = ''
    
    def get_user_token(self):
        if self.user_token == '':
            raise Et(2, "请先设置user_token")
        return self.user_token
    
    def set_user_token(self, user_token):
        self.user_token = user_token


    def get_host_name(self):
        # 后面可以改成从配置中心获取
        PROJECT_ENV = os.environ.get("PROJECT_ENV", "dev")
        if PROJECT_ENV == "dev": # 本地测试
            endpoint_url = "http://localhost:8000"
        else:
            endpoint_url = "http://tfduck.163py.com"   # 后面可以改成从配置中心获取
        return {'s': 1, 'v': endpoint_url}  
    
    # 旧的接口也从tfduck的外部接口调用

    def get_model_data(self, app_name, model_name, query_str, r_field):
        """
        @des: 获取模型数据
        """
        hresult = self.get_host_name()
        if hresult['s'] == 1:
            host_name = hresult['v']
        else:
            return hresult
        data = {
                "tfduck_api_token": self.get_user_token(),
                "app_name": app_name,
                "model_name": model_name,
                "query_str": query_str,
                "r_field": r_field,
        }
        #
        url = os.path.join(host_name, 'get_model_data')
        for i in range(self.retry_count):
            try:
                res = requests.post(url, json=data, timeout=self.r_timeout)
                break
            except:
                time.sleep(self.retry_delay)
                continue
        else:  # 循环完成
            raise Et(
                2, f"bmobj clean set_model_data retry {self.retry_count} times failed!")
        #
        res = res.json()
        if res['s'] == 1:
            result = res['data']
            return result
        else:
            raise Et(2, res['msg'])
        
    
    def set_model_data(self, app_name, model_name, query_str, r_field, save_data):
        """
        @des: 给模型更新数据
        """
        hresult = self.get_host_name()
        if hresult['s'] == 1:
            host_name = hresult['v']
        else:
            return hresult
        data = {
                "tfduck_api_token": self.get_user_token(),
                "app_name": app_name,
                "model_name": model_name,
                "query_str": query_str,
                "r_field": r_field,
                "save_data": save_data
        }
        #
        url = os.path.join(host_name, 'set_model_data')
        for i in range(self.retry_count):
            try:
                res = requests.post(url, json=data, timeout=self.r_timeout)
                break
            except:
                time.sleep(self.retry_delay)
                continue
        else:  # 循环完成
            raise Et(
                2, f"bmobj clean set_model_data retry {self.retry_count} times failed!")
        #
        res = res.json()
        if res['s'] == 1:
            result = res['data']
            return result
        else:
            raise Et(2, res['msg'])

    
    def run_model_method(self, app_name, model_name, run_method, run_params):
        """
        @des: 运行模型自定义方法
        """
        hresult = self.get_host_name()
        if hresult['s'] == 1:
            host_name = hresult['v']
        else:
            return hresult
        data = {
            "tfduck_api_token": self.get_user_token(),
            "app_name": app_name,
            "model_name": model_name,
            "run_method": run_method,
            "run_params": run_params
        }
        #
        url = os.path.join(host_name, 'run_model_method')
        for i in range(self.retry_count):
            try:
                res = requests.post(url, json=data, timeout=self.r_timeout)
                break
            except:
                time.sleep(self.retry_delay)
                continue
        else:  # 循环完成
            raise Et(
                2, f"bmobj clean retry {self.retry_count} times failed!")
        #
        res = res.json()
        if res['s'] == 1:
            result = res['data']
            return result
        else:
            raise Et(2, res['msg'])



    # start----tfduck的外部接口调用
    def run_sptasks_by_taskname(self, task_names):
        """
        @des: http外部接口运行非定时的任务
        一般用于多个工程的上游依赖，A->B
        当A工程执行完毕后, 执行B工程的所有任务
        @param user_token[必填]: tfduck的用户token; self.set_user_token('xxxxx')
               task_names[必填]: 运行的sptask任务(不是任务工程，也不是任务记录)的全名的列表, 比如 ["同步任务_3",...]
        @return: 
               成功---{"s":1,  'value':[任务名称]}
               失败---{"s: 非1, 'msg':'失败原因'}
        @example:
        ######
        from tfduck.common.defines import BMOBJ
        BMOBJ.http_api.run_sptasks_by_taskname('xxxxx', ['helloworld_end_tf_2_0'])
        ######
        """
        hresult = self.get_host_name()
        if hresult['s'] == 1:
            host_name = hresult['v']
        else:
            return hresult
        #
        url = os.path.join(host_name, 'run_sptasks_by_taskname')
        data = {
            'user_token': self.get_user_token(),
            'task_names': json.dumps(task_names),
        }
        res = requests.post(url, data=data, timeout=(5, 30))
        result = res.json()
        return result

    def run_sptasks_by_projectname(self, project_names):
        """
        @des: http外部接口运行非定时的任务
        一般用于多个工程的上游依赖，A->B
        当A工程执行完毕后, 执行B工程的所有任务
        @param user_token   [必填]: tfduck的用户token; self.set_user_token('xxxxx')
               project_names[必填]: 运行的sptask任务工程的名称(不是任务，也不是任务记录)的全名的列表, 比如 ["同步任务工程名",...]
        @return: 
               成功---{"s":1,  'value':[任务名称]}
               失败---{"s: 非1, 'msg':'失败原因'}
        @example:
        ######
        from tfduck.common.defines import BMOBJ
        BMOBJ.http_api.run_sptasks_by_projectname('xxxxx', ['helloworld_end_tf_2_0'])
        ######
        """
        hresult = self.get_host_name()
        if hresult['s'] == 1:
            host_name = hresult['v']
        else:
            return hresult
        #
        url = os.path.join(host_name, 'run_sptasks_by_projectname')
        data = {
            'user_token': self.get_user_token(),
            'project_names': json.dumps(project_names)
        }
        res = requests.post(url, data=data, timeout=(5, 30))
        result = res.json()
        return result

    def get_sptasks_info_by_taskname(self, task_names, start_time, end_time):
        """
        @des: http外部接口查询任务的信息，根据任务名称查询
        一般用于多个工程的上游依赖，A->B
        当A工程执行完毕后, 执行B工程的所有任务
        @param user_token[必填]: tfduck的用户token;self.set_user_token('xxxxx')
               task_names[必填]: 运行的sptask任务(不是任务工程，也不是任务记录)的全名的列表, 比如 ["同步任务_3",...]
               start_time[选填]: 任务的create_time的查询时间范围的开始时间，格式YYYY-MM-DD HH:mm:ss 例如 2022-02-15 14:23:44
               end_time  [选填]: 任务的create_time的查询时间范围的结束时间，格式YYYY-MM-DD HH:mm:ss 例如 2022-02-15 18:23:44
        @return: 
               成功---{"s":1,  'value':[{'task__name': '任务名称', 'task__id': 109, 'id': 78, 'state': 3, 'revoke_state': 1},...]}
               失败---{"s: 非1, 'msg':'失败原因'}
        @example:
        ######
        from tfduck.common.defines import BMOBJ
        BMOBJ.http_api.get_sptasks_info_by_taskname('xxxxx', ['helloworld_end_tf_2_0'], '2022-02-15 14:23:44', '2022-02-15 18:23:44')
        ######
        """
        hresult = self.get_host_name()
        if hresult['s'] == 1:
            host_name = hresult['v']
        else:
            return hresult
        #
        url = os.path.join(host_name, 'get_sptasks_info_by_taskname')
        data = {
            'user_token': self.get_user_token(),
            'task_names': json.dumps(task_names),
            'start_time': start_time,
            'end_time': end_time,
        }
        res = requests.post(url, data=data, timeout=(5, 30))
        result = res.json()
        return result

    def get_sptasks_info_by_projectname(self, project_names, start_time, end_time):
        """
        @des: http外部接口查询任务的信息，根据任务工程名称查询
        一般用于多个工程的上游依赖，A->B
        当A工程执行完毕后, 执行B工程的所有任务
        @param user_token[必填]: tfduck的用户token;self.set_user_token('xxxxx')
               project_names[必填]: 运行的sptask任务(不是任务工程，也不是任务记录)的全名的列表, 比如 ["同步任务_3",...]
               start_time[选填]: 任务的create_time的查询时间范围的开始时间，格式YYYY-MM-DD HH:mm:ss 例如 2022-02-15 14:23:44
               end_time  [选填]: 任务的create_time的查询时间范围的结束时间，格式YYYY-MM-DD HH:mm:ss 例如 2022-02-15 18:23:44
        @return: 
               成功---{"s":1,  'value':[{'task__name': '任务名称', 'task__id': 109, 'id': 78, 'state': 3, 'revoke_state': 1},...]}
               失败---{"s: 非1, 'msg':'失败原因'}
        @example:
        ######
        from tfduck.common.defines import BMOBJ
        BMOBJ.http_api.get_sptasks_info_by_projectname('xxxxx', ['helloworld_end_tf_2_0'], '2022-02-15 14:23:44', '2022-02-15 18:23:44')
        ######
        """
        hresult = self.get_host_name()
        if hresult['s'] == 1:
            host_name = hresult['v']
        else:
            return hresult
        #
        url = os.path.join(host_name, 'get_sptasks_info_by_projectname')
        data = {
            'user_token': self.get_user_token(),
            'project_names': json.dumps(project_names),
            'start_time': start_time,
            'end_time': end_time,
        }
        res = requests.post(url, data=data, timeout=(5, 30))
        result = res.json()
        return result

    # end----tfduck的外部接口调用


BMOBJ = BaseMethod()



"""
auhtor: yuanxiao
datetime: 20210423
python3.8
pyspark 提交阿里云ask集群管理器
pip install kubernetes==12.0.1
"""
import json
from tfduck.common.defines import BMOBJ, Et
from tfduck.oss.oss import AliyunOss
from kubernetes import client, config
import uuid
import arrow
import os
import base64
import subprocess


class K8sManage(object):
    """
    """

    def __init__(self, ctx, k8s_conn_config="~/.kube/config"):
        """
        """
        """
        全局初始化
        """
        self.ctx = ctx
        self.k8s_conn_config = k8s_conn_config
        """
        生成k8s客户端
        """
        self.get_k8s_client()

    def get_k8s_client(self):
        """
        @des: 获取k8s操作客户端
        例子  https://github.com/kubernetes-client/python/tree/master/examples
        api文档 https://github.com/kubernetes-client/python/blob/master/kubernetes/README.md
        """
        config.load_kube_config(self.k8s_conn_config)
        v1 = client.CoreV1Api()
        self.k8s_client = v1
        # print("Listing pods with their IPs:")
        # ret = v1.list_pod_for_all_namespaces(watch=False)
        # for i in ret.items:
        #     print("%s\t%s\t%s" %
        #           (i.status.pod_ip, i.metadata.namespace, i.metadata.name))

    def get_all_pods(self):
        """
        @des: 获取所有的k8s的pod, 默认命名空间的
        """
        # print("Listing pods with their IPs:")
        ret = self.k8s_client.list_pod_for_all_namespaces(watch=False)
        pod_infos = []
        for i in ret.items:
            if i.metadata.namespace == "default":
                pod_infos.append({'name': i.metadata.name,
                                  'status': i.status.phase,
                                  'create_time': i.metadata.creation_timestamp})
        return pod_infos

    def get_pod_info(self, pod_name):
        """
        @des: 获取pod的基础信息，状态等信息
        """
        # resp = self.k8s_client.read_namespaced_pod_status(
        #     name="tfspark-test-7200-driver-1e5ab8f2a41d11eb93c3acde48001122", namespace='default')
        # resp = self.k8s_client.read_namespaced_pod(
        #     name="tfspark-test-7200-driver-1e5ab8f2a41d11eb93c3acde48001122", namespace='default')
        pod_obj = self.k8s_client.read_namespaced_pod(
            name=pod_name, namespace='default')
        # print(resp)
        # print(resp.metadata.name, resp.status.phase)
        # return resp.status.phase # Pending,Succeeded, Running, Failed
        return pod_obj

    def get_pod_log(self, pod_name, lines_count=1000):
        """
        @des: 获取pod的日志--测试成功
        """
        # print(help(self.k8s_client.read_namespaced_pod_log))
        # log_content = self.k8s_client.read_namespaced_pod_log(
        #     'tfspark-test-7200-driver-c697aadea41d11eb9d2bacde48001122', 'default', pretty=True, tail_lines=1)  # container='spark-kubernetes-driver'
        log_content = self.k8s_client.read_namespaced_pod_log(
            pod_name, 'default', pretty=True, tail_lines=lines_count)  # tail_lines 表示从结尾开始获取日志的行数
        # print(log_content)
        return log_content

    # def patch_pod_info(self, pod_name):
    #     """
    #     @des: 修改pod的信息，包括状态
    #     """
    #     pod_obj = self.get_pod_info(pod_name)
    #     pod_obj.status.phase = ""
    #     # print(help(self.k8s_client.patch_namespaced_pod))
    #     self.k8s_client.patch_namespaced_pod(pod_name, 'default', pod_obj)

    def delete_pod(self, pod_name):
        """
        @des: 删除pod
        delete是删除---
        delete_namespaced_pod
        """
        # print(help(self.k8s_client.delete_namespaced_pod))
        # pod_obj = self.get_pod_info(pod_name)
        # print(pod_obj.metadata.name, pod_obj.metadata.creation_timestamp, pod_obj.status.phase)
        # pod_create = pod_obj.metadata.creation_timestamp
        # status = pod_obj.status.phase
        r = self.k8s_client.delete_namespaced_pod(pod_name, 'default')
        return r


if __name__ == "__main__":
    sm = K8sManage(ctx={}, k8s_conn_config="~/.kube/config")
    print(sm.get_pod_info("tfspark-test111-7200-driver-d883cda4a63411ebb0e70242ac110002").status.phase)
    # sm.get_pod_log("tfspark-test-7200-driver-3b7d45b8a57011eb8accacde48001122")
    # print(sm.delete_pod("tfspark-test-7200-driver-d4d431fea57511eb86e3acde48001122"))
    # print(sm.get_all_pods())



"""
封装一些基础方法
"""


class BaseTga(object):
    def __init__(self):
        pass

    def tran_dt_by_zone(self, field_name, field_zone_offset, target_offset=0):
        """
        #zone_offset是#event_time当对于utc时区的偏移量
        @des: 调用
        tran_zone_sql = cobj.tran_zone_sql(a."#event_time", a."#zone_offset", 0)  # 统一转为utc
        例如：
            # 加载
            from hv_user_predit_tf_2.src.common.config import Config as GAConfig
            gconf = GAConfig.getInstance()
            # 日期分为part_date的范围和event_time的范围， part_date的范围适当取大
            part_date = '2020-12-01'
            et_start = arrow.get(part_date).format("YYYY-MM-DD")
            et_end = arrow.get(part_date).shift(days=1).format("YYYY-MM-DD")
            #---------------
            SELECT
                *
            FROM (
                SELECT
                    "#user_id",
                    {gconf.tran_dt_by_zone("#event_time", "#zone_offset", 0)} as event_time_utc,
                    "$part_event"
                FROM
                    v_event_64
                WHERE
                    "$part_event" = 'new_device'
                    ---- "$part_date"的取值必须比event_time的范围大2天，因为有时区的问题
                    AND "$part_date" >= '{arrow.get(et_start).shift(days=-2).format("YYYY-MM-DD")}'
                    AND "$part_date" <= '{arrow.get(et_end).shift(days=2).format("YYYY-MM-DD")}'
            )
            WHERE
                event_time_utc >= timestamp '{et_start}'
                AND event_time_utc < timestamp '{et_end}'
                -- AND to_unixtime(current_timestamp)-to_unixtime(event_time_utc) >= 43200 --注册时间必须大于12小时【可选，模型会根据数据去判断是否是优质用户，如果第一次预测不是，可能第二次预测就是了】
        """
        sql = f"""
if(
    "{field_zone_offset}" is not null,
    date_add(
        'second',
        cast((0 - "{field_zone_offset}") * 3600 + {target_offset*3600}  as integer),
        "{field_name}"
    ),
    "{field_name}"
) 
"""
        return sql



'''
@des: tga获取批量转换特征值数据拉取的sql基类


准备工作: 
--------------------------------------
gconf的属性有这些:
########
   part_date_start  # 特征值取新增用户的开始日期, 例如"2022-05-20", 字符串类型
   part_date_end    # 特征值取新增用户的结束日期, 例如"2022-05-25", 字符串类型
   level_durs       # 特征取值段，由小到大排列, 例如 [43200, 43200*2], 这里是秒为单位，整型数组；多少时间段发生事件作为特征值
   tga_user_table   # tga用户表名, 例如 "v_user_7", 字符串
   tga_event_table  # tga事件表名, 例如 "v_event_7", 字符串 
########

如何使用(需要自己定义一个子类继承这个类， 代码如下): 
#######
from magic_number_train_tf_2.src.common.config import Config as GAConfig # 这里换成你自己的项目
from tfduck.tga.predict_sql_yh import PredictFeatureSql

def call_method(self, ctx=None, **kwargs):
    """
    @des: 每个具体项目的训练特征数据拉取 
    """
    gconf = GAConfig.getInstance()  # 保证gconf有上面准备工作的属性
    """
    设置项目属性-----下面的都需要自己配置------
    """
    # 需要的事件
    need_events = (
        'new_device', 'new_session', 'act_level_path'
    )
    # 需要的属性
    need_event_attrs = (
        "#lib", "#country_code", "$part_event", "#user_id",  "sdk_session_time",
        "object_type", "act", "object_number", "act_object",
    )
    # 特征值名称(不要用字母a作为key)---
    feature_names = {
        'b': '常规关卡通关',
        'c': '冒险关卡通关'
    }
    # 特征值sql---模板保持不变，变里面的内容即可---
    sub_sql_fs = {
        'b':            """
                        --%(real_des)s
                        sum(
                            if(
                            "$part_event"='act_level_path' and object_type='normal' and act='win' and rt_dur<%(level_dur)s,
                            1, 
                            0)
                        ) as %(sub_field_name)s
                        """,
        'c':            """
                        --%(real_des)s
                        sum(
                            if(
                            "$part_event"='act_level_path' and object_type='adventure' and act='win' and rt_dur<%(level_dur)s,
                            1, 
                            0)
                        ) as %(sub_field_name)s
                        """
    }       
    # 额外的event表属性和预查询的用户属性base_user_table_fields里面选，比如(可选参数)
    addon_attrs = ["#screen_width", "#screen_height"]     
    # 是否等待用户注册后经过max(gconf.level_durs)的时间才开始计算特征值(可选参数)---默认是不等待
    wait_dur_get_feature    
    """
    创建调用实例---最好通过dict创建实例
    """         
    pf_sql_obj = PredictFeatureSql(
            ctx = ctx, 
            gconf = gconf, 
            need_events = need_events, 
            need_event_attrs = need_event_attrs, 
            feature_names = feature_names,
            sub_sql_fs = sub_sql_fs,
            addon_attrs = addon_attrs # 可选参数
    )  
    sql = pf_sql_obj.get_sql() # 这个sql就是拉取特征值的sql
#######
'''
from tfduck.common.defines import BMOBJ, Et
from tfduck.tga.predict_sql_retain import PredictFeatureSql as BasePredictFeatureSql
import arrow


class PredictFeatureSql(BasePredictFeatureSql):
    """
    @des:sql批量转换模板基类
    """
    pass


import sys
import getopt
import requests
import zipfile
import os
import shutil
import base64
import uuid
import importlib
from tfduck.common.defines import BMOBJ


def remove_file(file_path):
    try:
        os.remove(file_path)
    except:
        pass

def set_host(host):
    """
    @des: 将host存入文件
    """
    if host != "del":
        if host.find("http://") == -1 and host.find("https://") == -1:
            print("设置失败: host 应该带有http或者https头")
            return
        if not host.endswith("/"):
            host = f"{host}/"
    base_dir = os.path.join(os.environ.get('HOME', ''), "tmp/tfduck/")
    apihost_file = os.path.join(base_dir, "apihost_file")
    if host != 'del':
        remove_file(apihost_file)
        if not os.path.exists(base_dir):
            os.makedirs(base_dir)
        with open(apihost_file, 'w') as f:
            f.write(BMOBJ.jsondumps({'host': host}))
        print("sethost成功")
    else:
        remove_file(apihost_file)
        print("删除host成功")


def get_host():
    base_dir = os.path.join(os.environ.get('HOME', ''), "tmp/tfduck/")
    apihost_file = os.path.join(base_dir, "apihost_file")
    if os.path.exists(apihost_file):
        with open(apihost_file, 'r') as f:
            data = BMOBJ.jsonloads(f.read())
        host = data['host']
        return f"{host}"
    else:
        return f"http://tfduck.talefun.com/"


def get_combin_host(rpath):
    host = get_host()
    return f"{host}{rpath}"


# def get_project_id():
#     current_path = os.path.abspath("./")
#     parent_path = os.path.dirname(current_path)
#     project_id = check_project(current_path)
#     return project_id

def find_package_name(project_path):
    """
    @des: 寻找包
    """
    project_path = os.path.abspath(project_path)
    top_package_name = os.path.basename(project_path)
    sub_package_name = None
    for sub_file in os.listdir(project_path):
        if os.path.isdir(sub_file) and sub_file.startswith(top_package_name):
            sub_package_name = sub_file
            break
    if sub_package_name is None:
        raise Exception("同步路径错误, 子包不存在，同步路径应该在工程文件夹内")
    return (top_package_name, sub_package_name)


def check_project(project_path):
    #
    _, sub_package_name = find_package_name(project_path)
    #
    config_path = os.path.join(project_path, f"{sub_package_name}/config.py")
    dag_path = os.path.join(project_path, f"{sub_package_name}/dag.py")
    if not os.path.exists(dag_path) or not os.path.exists(config_path):
        raise Exception("同步路径错误，同步路径应该在工程文件夹内")
    # 获取 uuid
    config_path = os.path.join(project_path, f"{sub_package_name}/config.py")
    with open(config_path, 'rb') as r:
        compile_obj = compile(r.read(), "", 'exec')
    ns = {}
    exec(compile_obj, ns, ns)
    config = ns["config"]
    project_id = config.get("pid", None)
    if not project_id:
        raise Exception("工程配置pid不存在")
    return project_id


def compress(folder_path="./"):
    """
    @des:-------------------------------------------------------
    import shutil
    shutil.make_archive(base_name, format, root_dir, base_dir)
    base_name : 创建的目标文件名，包括路径，减去任何特定格式的扩展。
    format : 压缩包格式。”zip”, “tar”, “bztar”或”gztar”中的一个。
    root_dir : 打包时切换到的根路径。也就是说，开始打包前，会先执行路径切换，切换到root_dir所指定的路径。默认值为当前路径
    base_dir : 开始打包的路径。也就是说，该命令会对base_dir所指定的路径进行打包，默认值为 root_dir ，即打包切换后的当前目录。亦可指定某一特定子目录，从而实现打包的文件包含此统一的前缀路径
    owner 和 group 为创建tar包时使用，默认为用户当前的 owner & group
    @des:----------------------------------
    unpack_archive(filename, extract_dir=None, format=None)： 解压操作。Python3新增方法
    filename：文件路径
    extract_dir：解压至的文件夹路径。文件夹可以不存在，会自动生成
    format：解压格式，默认为None，会根据扩展名自动选择解压格式
    """
    # 目录验证
    current_path = os.path.abspath(folder_path)
    parent_path = os.path.dirname(current_path)
    project_id = check_project(current_path)
    #
    project_name = os.path.basename(current_path)
    zip_file_path = os.path.join(parent_path, f"{project_name}")
    if 0:
        # 这样即可实现目录下所有文件，但不打包目录，
        # 一般配合shutil.unpack_archive(filename, "/你指定的解压目录", "zip") 来使用
        real_file_path = shutil.make_archive(
            zip_file_path, "zip", current_path, f"./")
    else:
        # 这样即可实现打包目录，解压后会生成目录
        # 一般配合shutil.unpack_archive(filename, "/你指定的解压目录的父目录", "zip") 来使用
        real_file_path = shutil.make_archive(
            zip_file_path, "zip", parent_path, f"./{project_name}")
    # print(real_file_path)
    return real_file_path, project_id


def sync():
    auth_info = get_auth_info()
    username = auth_info.get("username", '')
    password = auth_info.get('password', '')
    if not username or not password:
        print("你还未进行认证，请先执行[tfduck login -uUSERNAME -pPASSWORD]命令")
        return
    real_file_path, project_id = compress("./")
    try:
        with open(real_file_path, "rb") as r:
            b64_content = base64.b64encode(r.read()).decode()
            # content = base64.b64decode(b64_content) # django view 通过这个解码即可得到二进制数据
    finally:
        remove_file(real_file_path)
    #
    _, sub_package_name = find_package_name("./")
    with open(f"./{sub_package_name}/config.py", 'rb') as r:
        compile_obj = compile(r.read(), "", 'exec')
    ns = {}
    exec(compile_obj, ns, ns)
    _config = ns["config"]
    #
    ptype = _config.get("ptype", None)
    if ptype == "retask":
        rurl = get_combin_host("syncreproject")
    elif ptype == "sptask":
        rurl = get_combin_host("syncspproject")
    else:
        raise Exception("ptype not in retask, sptask")
    #
    data = {"username": username, "password": password, "is_clean": False,
            "project": b64_content, "project_id": project_id}
    res = requests.post(url=rurl,
                        data=data, timeout=(10, 300))
    try:
        print(res.json()['result'])
    except:
        print(res.text)


def init():
    auth_info = get_auth_info()
    username = auth_info.get("username", '')
    password = auth_info.get('password', '')
    if not username or not password:
        print("你还未进行认证，请先执行[tfduck login -uUSERNAME -pPASSWORD]命令")
        return
    real_file_path, project_id = compress("./")
    try:
        with open(real_file_path, "rb") as r:
            b64_content = base64.b64encode(r.read()).decode()
            # content = base64.b64decode(b64_content) # django view 通过这个解码即可得到二进制数据
    finally:
        remove_file(real_file_path)
    #
    _, sub_package_name = find_package_name("./")
    with open(f"./{sub_package_name}/config.py", 'rb') as r:
        compile_obj = compile(r.read(), "", 'exec')
    ns = {}
    exec(compile_obj, ns, ns)
    _config = ns["config"]
    #
    ptype = _config.get("ptype", None)
    if ptype == "retask":
        rurl = get_combin_host("syncreproject")
    elif ptype == "sptask":
        rurl = get_combin_host("syncspproject")
    else:
        raise Exception("ptype not in retask, sptask")
    #
    data = {"username": username, "password": password, "is_init": True,
            "project": b64_content, "project_id": project_id}
    res = requests.post(url=rurl,
                        data=data, timeout=(10, 300))
    try:
        print(res.json()['result'])
    except:
        print(res.text)


def create_project(create_project_type, package_name):
    """
    @des: 创建工程模板
    """
    auth_info = get_auth_info()
    username = auth_info.get("username", '')
    password = auth_info.get('password', '')
    if not username or not password:
        print("你还未进行认证，请先执行[tfduck login -uUSERNAME -pPASSWORD]命令")
        return
    # 判断工程是否存在
    if os.path.exists(f"./{package_name}"):
        print(f"工程名: {package_name} 已经存在, 请确认删除后再创建！")
        return
    #
    if create_project_type == "retask":
        rurl = get_combin_host("create_re_project")
    elif create_project_type == "sptask":
        rurl = get_combin_host("create_sp_project")
    else:
        raise Exception("create_project_type not in retask, sptask")
    data = {"username": username, "password": password,
            "package_name": package_name}
    res = requests.post(url=rurl,
                        data=data, timeout=(10, 60))
    res_data = res.json()
    if res_data.get("s", 2) == 1:
        # res_package_name = res_data["package_name"]
        file_data = res_data["file_data"]
        file_content = base64.b64decode(file_data)
        #
        try:
            tmp_file_path = f"./{BMOBJ.get_unique_id2()}.zip"
            with open(tmp_file_path, "wb") as f:
                f.write(file_content)
            project_path = f"./{package_name}"
            shutil.unpack_archive(
                tmp_file_path, extract_dir=project_path, format='zip')
        finally:
            remove_file(tmp_file_path)
        print("create project success")
    else:
        print(res_data.get('msg', "未知错误"))


def login(username, password):
    rurl = get_combin_host("tf_authentication")
    data = {'username': username, 'password': password}
    res = requests.post(url=rurl,
                        data=data, timeout=(10, 60))
    res_data = res.json()
    if res_data['s'] == 1:
        base_dir = os.path.join(os.environ.get('HOME', ''), "tmp/tfduck/auth")
        session_file = os.path.join(base_dir, "session_file")
        remove_file(session_file)
        if not os.path.exists(base_dir):
            os.makedirs(base_dir)
        with open(session_file, 'w') as f:
            f.write(BMOBJ.jsondumps(data))
        print("认证成功")
    else:
        print(f"认证失败: {res_data.get('msg', '')}")


def logout():
    base_dir = os.path.join(os.environ.get('HOME', ''), "tmp/tfduck/auth")
    session_file = os.path.join(base_dir, "session_file")
    remove_file(session_file)
    print(f"退出登录成功")


def get_auth_info():
    base_dir = os.path.join(os.environ.get('HOME', ''), "tmp/tfduck/auth")
    session_file = os.path.join(base_dir, "session_file")
    try:
        with open(session_file, 'r') as f:
            data = f.read()
    except:
        data = "{}"
    real_data = BMOBJ.jsonloads(data)
    return real_data


def main(*args, **kwargs):
    help_doc = """
    -h 查看帮助文件
    -v 查看版本
    -m 配合create命令使用，有该参数则是创建retask，否则都是sptask,测试用，用户可以不用管这个
    { -u 用户名 -p 密码 login }     认证
    { info }                      查看当前登录用户
    { create [project name]  }    创建工程(不要取和系统工程同名的名称，比如pandas，django等名称)
    { sync }                      cd到工程目录内执行该命令,同步工程到服务器
    { init }                      cd到工程目录内执行该命令, 同步工程到服务器； 删除原来的dags,执行记录和作业调度并重新创建
    { sethost [api的host地址或者del]}     执行该命令，可以切换 host地址；如果是del，则删除host文件，使用默认；管理员使用，用户可以不用管这个
    ##################################################
    example:
    认证:
    tfduck -uUSERNAE -pPASSWORD login 
    创建工程
    tfduck create helloworld
    初始化当前工程
    tfduck init
    如果工程只修改了src源码，并没有修改dag.py，则可执行
    tfduck sync
    """
    # print(1111)
    # print(args)
    # print(kwargs)
    #
    opts, args = getopt.getopt(
        args, "hvu:p:d:m", ["sync", "create=", "help", "login", "logout", "info", "sethost="])
    # print(opts, args)
    try:
        username = None
        password = None
        is_sync = False
        is_init = False
        is_create = False
        is_login = False
        is_logout = False
        is_info = False
        is_sethost = False
        create_project_type = "sptask"
        for op, value in opts:
            if op == "-h":
                print(help_doc)
            elif op == "-v":
                import tfduck
                print(tfduck.__version__)
            elif op == "-u":
                username = value
            elif op == "-p":
                password = value
            elif op == "-m":
                create_project_type = 'retask'
        try:
            arg = args[0]
        except:
            arg = None
        try:
            arg_value = args[1]
        except:
            arg_value = None
        #
        if arg == "sync":
            is_sync = True
        elif arg == "init":
            is_init = True
        elif arg == "create":
            is_create = True
            if not arg_value:
                raise Exception("create命令必须有一个工程名参数")
        elif arg == "login":
            is_login = True
        elif arg == "logout":
            is_logout = True
        elif arg == "info":
            is_info = True
        elif arg == "sethost":
            is_sethost = True
            if not arg_value:
                raise Exception("sethost命令必须有一个参数")
        #
        if is_login:
            login(username, password)
        elif is_logout:
            logout()
        elif is_create:
            create_project(create_project_type, arg_value)
        elif is_sync:
            sync()
        elif is_init:
            init()
        elif is_sethost:
            set_host(arg_value)
        elif is_info:
            auth_info = get_auth_info()
            username = auth_info.get("username", '')
            host = get_host()
            info = f"""
当前登录用户: {username if username else "未认证"}
当前host: {host}
            """
            print(info)
    except Exception as e:
        print(e)
        print(help_doc)


if __name__ == "__main__":
    args = sys.argv[1:]
    main(*args)



"""
s3的公共操作
版本号见setup.py
"""
# coding=utf-8


class SAOper(object):
    """
    @des: S3的公共操作, 上传文件
    """

    def __init__(self, ctx={}):
        """
        @des: 初始化
        """
        self.ctx = ctx

    def submit_proccess_task(self,
                             sagemaker_session,
                             base_job_name,
                             image_uri,
                             role,
                             #
                             code,
                             arguments,
                             #
                             instance_count=1,
                             command=['/opt/program/submit'],
                             instance_type='ml.c5.xlarge',
                             max_runtime_in_seconds=86400,
                             env={'mode': 'python'},
                             #
                             logs=False,
                             wait=True,
                             retry_count=3  # 出错后重试次数
                             ):
        '''
        @des: 提交处理任务--例如特征值处理任务，特征值工程到sagemaker
        参考代码:
        ####
        def call_tzz_etl(self):
            ctx = self.ctx
            gconf = GAConfig.getInstance()
            from sagemaker import get_execution_role
            import sagemaker
            #
            s3_session = boto3.Session(
                aws_access_key_id=self.access_key,
                aws_secret_access_key=self.secret_key,
                region_name='us-east-2'
            )
            #
            sagemaker_session = sagemaker.Session(boto_session=s3_session)
            iam = s3_session.client('iam')
            role = iam.get_role(
                RoleName='AmazonSageMaker-ExecutionRole-20200817T105993')['Role']['Arn']
            #
            from sagemaker.sklearn.processing import SKLearnProcessor, ScriptProcessor
            from sagemaker.processing import ProcessingInput, ProcessingOutput
            """
            获取我自己创建的容器镜像资源
            """
            account_id = s3_session.client(
                'sts').get_caller_identity().get('Account')
            BMOBJ.clog(ctx, 111, self.part_date, account_id)
            # region = boto3.session.Session().region_name
            region = s3_session.region_name
            ecr_repository = 'sagemaker-spark-yx'
            tag = ':latest'
            uri_suffix = 'amazonaws.com'
            if region in ['cn-north-1', 'cn-northwest-1']:
                uri_suffix = 'amazonaws.com.cn'
            spark_repository_uri = '{}.dkr.ecr.{}.{}/{}'.format(
                account_id, region, uri_suffix, ecr_repository + tag)
            BMOBJ.clog(ctx, spark_repository_uri)
            """
            开始跑测试脚本
            """
            spark_processor = ScriptProcessor(
                sagemaker_session=sagemaker_session,
                base_job_name='bf-raw-to-tzz',  # 不能用下划线
                image_uri=spark_repository_uri,
                command=['/opt/program/submit'],
                role=role,
                # 容器数量，这里有点疑问，如果是多个，怎么并发处理； 解答：是将数据做了shard放到了不同机器上
                # 不要在作业里面做模型训练和推理， 还是要起另外的训练任务和批量转换任务来做
                instance_count=1,  # 最多 nxlarge == n*4
                instance_type='ml.c5.xlarge',  # c类型的比较省钱,
                max_runtime_in_seconds=86400,
                env={'mode': 'python'})
            # bucket = sagemaker_session.default_bucket()
            code_path = BMOBJ.get_file_path(
                'bf_hv_user_predit/bf_hv_user_predit_tf_2/src/preprocess/preprocess_tzz.py')
            BMOBJ.clog(ctx, 111, self.bucket, code_path)
            input_prefix = gconf.get_s3_key(
                ["hv_user_predit_raw", self.part_date, "*"])
            output_prefix = gconf.get_s3_key(
                ["hv_user_predit_tzz", self.part_date])
            spark_processor.run(code=code_path,  # 在本地路径下面
                                arguments=['s3_input_bucket', self.bucket,
                                        's3_input_key_prefix', input_prefix,
                                        's3_output_bucket', self.bucket,
                                        's3_output_key_prefix', output_prefix],
                                logs=False, wait=True)
            # 清空原始数据表
            if 0: # 不用spark
                spark = gconf.get_spark()
                input_path = gconf.get_s3a_path(["hv_user_predit_raw", self.part_date])
                df_empty = spark.createDataFrame([(0,)], ["empty"])
                df_empty.write.csv(input_path, mode='overwrite', header=False)
            else:
                s3o = S3Oper(ctx)
                remote_path = gconf.get_s3_key(["hv_user_predit_raw", self.part_date])
                BMOBJ.clog(ctx, f"remote path {remote_path}")
                s3o.delete_folder(remote_path)
            #
            BMOBJ.clog(ctx, "end")
        ####
        '''
        from sagemaker.sklearn.processing import SKLearnProcessor, ScriptProcessor
        from sagemaker.processing import ProcessingInput, ProcessingOutput
        for i in range(retry_count):
            try:

                spark_processor = ScriptProcessor(
                    sagemaker_session=sagemaker_session,
                    base_job_name=base_job_name,  # 不能用下划线
                    image_uri=image_uri,
                    command=command,
                    role=role,
                    # 容器数量，这里有点疑问，如果是多个，怎么并发处理； 解答：是将数据做了shard放到了不同机器上
                    # 不要在作业里面做模型训练和推理， 还是要起另外的训练任务和批量转换任务来做
                    instance_count=instance_count,  # 最多 nxlarge == n*4
                    instance_type=instance_type,  # c类型的比较省钱,
                    max_runtime_in_seconds=max_runtime_in_seconds,
                    env=env)
                spark_processor.run(code=code,  # 在本地路径下面
                                    arguments=arguments,
                                    logs=logs, wait=wait)
                break
            except Exception as e:
                if str(e).find("Rate exceeded") != -1:
                    continue
                else:
                    raise e
        return True

    def submit_batch_tran_task(
        self,
        model_name,
        sagemaker_session,
        output_path,
        data,
        instance_count=1,
        instance_type='ml.m4.xlarge',
        accept="text/csv",
        assemble_with="Line",
        data_type='S3Prefix',
        content_type='text/csv',
        split_type='Line',
        input_filter="$[3:]",
        join_source="Input",
        output_filter="$",
        retry_count=3
    ):
        '''
        @des: 提交批量转换任务
        参考代码
        ######
        def tran(self):
        """
        @des: 批量转换--测试结果集
        """
        ctx = self.ctx
        BMOBJ.clog(ctx, f"start tran {self.part_date}")
        gconf = GAConfig.getInstance()
        s3_session = boto3.Session(
            aws_access_key_id=self.access_key,
            aws_secret_access_key=self.secret_key,
            region_name='us-east-2'
        )
        sagemaker_session = sagemaker.Session(boto_session=s3_session)
        # 预测的结果文件

        output_path = gconf.get_s3_path(
            ["hv_user_predit_tran", self.part_date, "tran"])
        output_path_s3a = gconf.get_s3a_path(
            ["hv_user_predit_tran", self.part_date, "tran"])
        output_path_key = gconf.get_s3_key(
            ["hv_user_predit_tran", self.part_date, "tran"])
        if 1:
            # 清空
            s3oper_obj = S3Oper(ctx)
            s3oper_obj.delete_folder(output_path_key)
            # 加强健壮性---特别注意train_data的路径前缀，不要和统计目录有混淆
            # 前缀一样会被当成一批，比如testsample111同级目录下有testsample111_xx都会被读到
            transformer = sagemaker.estimator.Transformer(model_name='bf-hv-user-train-model',
                                                          instance_count=1,
                                                          instance_type='ml.m4.xlarge',
                                                          sagemaker_session=sagemaker_session,
                                                          output_path=output_path,
                                                          accept="text/csv",   # 配合下面的join_source= "Input"
                                                          assemble_with="Line",  # 配合下面的join_source= "Input"
                                                          )
            tran_samples = gconf.get_s3_path(
                ["hv_user_predit_sample", self.part_date, 'predit'])
            tran_samples_3a = gconf.get_s3a_path(
                ["hv_user_predit_sample", self.part_date, 'predit'])
            BMOBJ.clog(ctx, tran_samples)
            #
            transformer.transform(data=tran_samples, data_type='S3Prefix',
                                  content_type='text/csv', split_type='Line',
                                  # 这里特别注意，$[a:b]是要取到b的和python的分片不太一样,所以这里不取最后一个元素，用-2
                                  # 不取第一列的用户id
                                  input_filter="$[3:]",
                                  join_source="Input",
                                  output_filter="$"
                                  )
            transformer.wait()
            BMOBJ.clog(ctx, "batch tran end")
        ######
        '''
        import sagemaker
        for i in range(retry_count):
            try:
                # 前缀一样会被当成一批，比如testsample111同级目录下有testsample111_xx都会被读到
                transformer = sagemaker.estimator.Transformer(model_name=model_name,
                                                              instance_count=instance_count,
                                                              instance_type=instance_type,
                                                              sagemaker_session=sagemaker_session,
                                                              output_path=output_path,
                                                              accept=accept,   # 配合下面的join_source= "Input"
                                                              assemble_with=assemble_with,  # 配合下面的join_source= "Input"
                                                              )
                #
                transformer.transform(data=data, data_type=data_type,
                                      content_type=content_type, split_type=split_type,
                                      # 这里特别注意，$[a:b]是要取到b的和python的分片不太一样,所以这里不取最后一个元素，用-2
                                      # 不取第一列的用户id
                                      input_filter=input_filter,
                                      join_source=join_source,
                                      output_filter=output_filter
                                      )
                transformer.wait()
                break
            except Exception as e:
                if str(e).find("Rate exceeded") != -1:
                    continue
                else:
                    raise e
        return True


if __name__ == '__main__':  # 打版本的时候一定记得记得脱敏
    pass
